﻿using System;
using System.Windows;
using System.Data;
using Infragistics.Controls.Charts;
using System.Windows.Controls;
using System.Windows.Data;
using Infragistics.Windows.Controls;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;

using System.Collections;
using System.Reflection;
using System.Data.OleDb;


namespace dynamicseries
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            
            #region "OlapXAxis"

            try
            {
                XamDataChart DataChart = new XamDataChart();
                
                string filename = "SampleData.xls";

                DataTable dtseries = new System.Data.DataTable();
                string Con = @"Provider=Microsoft.Jet.OLEDB.4.0;" +
                             @"Data Source=" + filename + ";" +
                             @"Extended Properties=" + Convert.ToChar(34).ToString() +
                             @"Excel 8.0;" + "Imex=2;" + "HDR=Yes;" + Convert.ToChar(34).ToString();

                OleDbConnection con = new OleDbConnection(Con);
                OleDbDataAdapter data = new OleDbDataAdapter("Select * from [Sheet1$]", con);
                data.Fill(dtseries);                

                DynamicTypeBuilder typeBuilder = new DynamicTypeBuilder
                {
                    DynamicAssemblyName = "MyAssembly",
                    DynamicTypeName = "Cube"
                };

                IList<DynamicTypePropertyInfo> properties = new List<DynamicTypePropertyInfo>();
                foreach (DataColumn column in dtseries.Columns)
                {
                    DynamicTypePropertyInfo propertyInfo = new DynamicTypePropertyInfo
                    {
                        PropertyName = column.ColumnName,
                        PropertyType = column.DataType
                    };
                    properties.Add(propertyInfo);
                }

                Type dynamicType = typeBuilder.GenerateType(properties);
                Type listType = typeof(List<>);
                Type genericListType = listType.MakeGenericType(dynamicType);
                IList list = (IList)Activator.CreateInstance(genericListType);
                foreach (DataRow dataRow in dtseries.Rows)
                {
                    object myDynamicInstance = Activator.CreateInstance(dynamicType);
                    foreach (DataColumn column in dtseries.Columns)
                    {
                        PropertyInfo propertyVal = dynamicType.GetProperty(column.ColumnName);
                        if (dataRow[column] != DBNull.Value)
                        {
                            propertyVal.SetValue(myDynamicInstance, dataRow[column], null);
                        }
                    }
                    list.Add(myDynamicInstance);
                }

                FlatDataSource flatDataSource = new FlatDataSource()
                {
                    ItemsSource = list,
                    Cube = DataSourceBase.GenerateInitialCube("Cube"),
                    Columns = DataSourceBase.GenerateInitialItems("[XField A].[XField A]"),
                    Rows = DataSourceBase.GenerateInitialItems("[XField B].[XField B]"),
                    Measures = DataSourceBase.GenerateInitialItems("Column 1,Column 2,Column 3,Column 4,Column 5,Column 6,Column 7,Column 8,Column 9,Column 10,Column 11,Column 12")
                };

                var cubeMetadata = new CubeMetadata();
                flatDataSource.CubesSettings.Add(cubeMetadata);

                foreach (DataColumn col in dtseries.Columns)
                {
                    //heirarchy
                    var hier = new HierarchyDescriptor
                    {
                        SourcePropertyName = col.ColumnName,
                    };
                    var allLevel = new HierarchyLevelDescriptor
                    {
                        LevelName = "All " + col.ColumnName + "(s)"
                    };
                    var entriesLevel = new HierarchyLevelDescriptor
                    {
                        LevelName = col.ColumnName,
                        LevelExpressionPath = col.ColumnName
                    };
                    hier.LevelDescriptors.Add(allLevel);
                    hier.LevelDescriptors.Add(entriesLevel);
                    flatDataSource.HierarchyDescriptors.Add(hier);

                    //if (col.DataType == typeof(double) || col.DataType == typeof(int) || col.DataType == typeof(Int16))
                    if (col.ColumnName.Contains("Column"))
                    {
                        var dm = new DimensionMetadata()
                        {
                            SourcePropertyName = col.ColumnName,
                            DisplayName = col.ColumnName,
                            DimensionType = DimensionType.Measure,
                            AggregatorType = AggregatorType.Sum,
                            DisplayFormat = "{0:N2}"
                        };
                        cubeMetadata.DimensionSettings.Add(dm);
                    }
                }

                NumericYAxis yAxis = new NumericYAxis();
                yAxis.MinimumValue = 0;
                yAxis.MaximumValue = 10000;
                OlapXAxis xAxis = new OlapXAxis();
                xAxis.Name = "xAxis";
                xAxis.DefaultSeries = OlapXAxisDefaultSeries.ColumnSeries;
                xAxis.DataSource = flatDataSource;
                xAxis.OlapAxisSource = OlapAxisSource.Rows;
                xAxis.YAxis = yAxis;
                DataChart.Axes.Add(xAxis);
                DataChart.Axes.Add(yAxis);

                this.grid1.Children.Add(DataChart);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

            #endregion "OlapXAxis"
        }
    }
}
